#include "lcd1202.h"
#include "config.h"

#include "font.h"

#include "fnt_libera_24x14.xbm"
// #include "fnt_terminus_24x14.xbm"

void lcd_init_gpio(void);
void lcd_reset(void);
void lcd_write_byte(unsigned char data, unsigned char rc);

static void lcd_set_row(unsigned char row);
static void lcd_set_col(unsigned char col);

void lcd_delay_ms(unsigned int ms);

struct {
    int inverse:1;
} flags;


/* hw undepended functions */

void lcd_init(void)
{
    lcd_init_gpio();
    lcd_reset();

	lcd_write_byte(0xE2, 0); // Reset
    lcd_delay_ms(10);
	lcd_write_byte(0xA4, 0); // Power saver off
	lcd_write_byte(0x2F, 0); // Power control set
	lcd_write_byte(0xAF, 0); // LCD display on

    lcd_clr();

    flags.inverse = -1;
}

void lcd_clr(void)
{
    int i;
    lcd_home();
    for(i=0; i<16*6*9; i++)
    {
        lcd_write_byte(0x00, 1);
    }
}

void lcd_big_char(unsigned char row, unsigned char col, unsigned char chr)
{
    // unsigned char* f0 = fnt_terminus_24x14_bits + chr*14*(24/8);
    unsigned char* f0 = fnt_libera_24x14_bits + chr*14*(24/8);
    unsigned char* f;
    unsigned char r, c;
	unsigned char ch;
    
    for(r = 0; r<3; r++)
    {
        f = f0 + r;
        lcd_set_row(row+r);
        lcd_set_col(col);
        for(c = 0; c<14; c++)
        {
            ch = *f;
            if(flags.inverse) ch = ch ^ 0xFF;
            lcd_write_byte(ch, 1);
            f += 3;
        }
    }
}

void lcd_pos(unsigned char row, unsigned char col)
{
    lcd_set_row(row);
    lcd_set_col(col*6);
}

void lcd_char(unsigned char c)
{
	unsigned char line;
	unsigned char ch;

    unsigned char* f = &(font6_8[c-32]);

	for(line=0;line<6;line++)
	{
		ch = *f++;
        if(flags.inverse) ch = ch ^ 0xFF;
		lcd_write_byte(ch, 1);
	}
}

void lcd_str(unsigned char* str)
{
    unsigned char c;
    while((c = *str) != 0)
    {
        lcd_char(c);
        str++;
    }
}

void lcd_inverse(char flag)
{
    flags.inverse = flag;
}

void lcd_reverse(unsigned char inv)
{
    if(inv)
        lcd_write_byte(0xA7, 0); // reverse display
    else
        lcd_write_byte(0xA6, 0); // normal display
}

void lcd_start_line(unsigned char line)
{
    lcd_write_byte(0x40 | (line & 0x3F), 0);
}

void lcd_home(void)
{
	lcd_write_byte(0xB0, 0); // Page address set
    lcd_write_byte(0x10, 0); // Sets the DDRAM column address - upper 3-bit
    lcd_write_byte(0x00, 0); // lower 4-bit
}

static void lcd_set_row(unsigned char row)
{
	lcd_write_byte(0xB0 | (row & 0x0F), 0); // Page address set
}

static void lcd_set_col(unsigned char col)
{
    lcd_write_byte(0x10 | (col>>4), 0); // Sets the DDRAM column address - upper 3-bit
    lcd_write_byte(0x00 | (col & 0x0F), 0); // lower 4-bit
}

/* MCU Hardware depend functions */
/* LPC2103 */
#include "utils.h"

#define lcd_rst_mark()
#define lcd_rst_release()

#define lcd_cs_mark() FIO0CLR = (1<<7)
#define lcd_cs_release() FIO0SET = (1<<7)

void lcd_delay_ms(unsigned int ms)
{
    delay_ms(ms);
}

void lcd_init_gpio(void)
{
#if (USEHW == 1)
    PINSEL0 |= (1<<8) | (1<<12);

    S0SPCR =
        (1<<2) | // BitEnable = 1
        (0<<3) | // CPHA = 0
        (0<<4) | // CPOL = 0
        (1<<5) | // Master
        (1<<6) | // LSB First = 0
        (9<<8) | // 9 bit per transfer
        0;
    S0SPCCR = 8;

    FIO0SET = (1<<7);
    FIO0DIR |= (1<<7);
#else
    FIO0SET = (1<<7);
    FIO0CLR = (1<<4) | (1<<6);
    FIO0DIR |= (1<<4) | (1<<6) | (1<<7);
#endif
}

void lcd_reset(void)
{
    lcd_rst_mark();
    lcd_delay_ms(10);
    lcd_rst_release();
    lcd_delay_ms(10);
}

void lcd_write_byte(unsigned char data, unsigned char rc)
{
    lcd_cs_mark();
#if (USEHW == 1)
    unsigned int d = data;
    if(rc) d |= 0x100;
    S0SPDR = d;
    while(!(S0SPSR & (1<<7))) // Wait while SPIF == 0
        ;
#else
    if(rc) FIO0SET = (1<<6);
      else FIO0CLR = (1<<6);
    FIO0SET = (1<<4);
    FIO0CLR = (1<<4);
    int i;
    for(i=0; i<8; i++)
    {
        if(data & 0x80) FIO0SET = (1<<6);
                   else FIO0CLR = (1<<6);
        data <<= 1;
        FIO0SET = (1<<4);
        FIO0CLR = (1<<4);
    }
#endif
    lcd_cs_release();
}

